import logging

from . import QueueBackend

try:
    from aiokafka import AIOKafkaProducer
except ImportError as ime:
    logging.warning(f"Required modules are not available: {ime.name}")
    raise


class KafkaBackend(QueueBackend):
    def __init__(self, kafka_uri: str, topic: str):
        self.kafka_producer = AIOKafkaProducer(bootstrap_servers=kafka_uri)
        self.topic = topic

    async def send_message(self, message: str):
        try:
            await self.kafka_producer.start()
            await self.kafka_producer.send(self.topic, message.encode("utf-8"))
        except Exception as e:
            logging.exception(e)
        finally:
            await self.kafka_producer.stop()



import logging
from typing import Optional

from . import QueueBackend

try:
    import mqttools
except ImportError:
    logging.warning("MQTT Library not found. Install using pip install asyncio-mqtt")
    raise


class MQTTBackend(QueueBackend):
    def __init__(self, topic: str,
                 hostname: str,
                 port: int = 1883,
                 # *,
                 username: Optional[str] = None,
                 password: Optional[str] = None,
                 ):
        self.topic = topic
        self.hostname = hostname
        self.port = port
        self.username = username
        self.password = password

    async def send_message(self, message: str):
        async with mqttools.Client(
                host=self.hostname,
                port=self.port,
                username=self.username,
                password=self.password,
        ) as client:
            client.publish(mqttools.Message(
                self.topic,
                message=message.encode()
            ))



import logging

from . import QueueBackend

try:
    import aioredis
except ImportError:
    logging.warning("Redis not found. Install using pip install aioredis")
    aioredis = None


class RedisBackend(QueueBackend):
    def __init__(self, redis_uri: str, db: int):
        self.redis_conn = aioredis.from_url(redis_uri, db=db)

    async def send_message(self, message):
        self.redis_conn.hset(message)



"""



author: irfanuddin
github: @iudeen
"""
import json
import logging
import typing
import asyncio

from starlette.middleware.base import DispatchFunction
from starlette.types import ASGIApp

from route_logger_middleware.backends import QueueBackend

logger = logging.getLogger("logging-middleware")

try:
    from fastapi import Response
    from starlette.middleware.base import BaseHTTPMiddleware
    from starlette.requests import Request
except ImportError:
    logging.warning("FastAPI not installed, import error. Run pip install fastapi")
    Response = BaseHTTPMiddleware = Request = None


class GlobalLoggerMiddleware(BaseHTTPMiddleware):
    def __init__(
            self, app: ASGIApp,
            dispatch: typing.Optional[DispatchFunction] = None,
            *,
            backend: QueueBackend,
            module_name: str
    ):
        super().__init__(app=app, dispatch=dispatch)
        self.backend = backend
        self.module_name = module_name

    async def dispatch(self, request: Request, call_next: typing.Callable) -> Response:
        receive_ = await request.receive()

        log_dict = {
            "module": self.module_name,
            "url": request.url.netloc,
            "path": request.url.path,
            "method": request.method,
            "cookies": dict(request.cookies),
            "path_params": request.path_params,
            "query_params": request.query_params.__dict__,
            "body": receive_.get('body').decode(),
            "client": request.client.host
        }
        loop = asyncio.get_event_loop()
        loop.create_task(self.backend.send_message(json.dumps(log_dict)))
        return await call_next(request)



from setuptools import setup

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='route-logger-fastapi-middleware',
    version='0.0.1',
    packages=['route_logger_middleware', 'route_logger_middleware.backends'],
    extras_require={
        "kafka": ["aiokafka>=0.7.2"],
        "mqtt": ["mqttools"],
        "redis": ["aioredis"]
    },
    url='https://github.com/iudeen/fastapi-route-logger',
    license='MIT',
    author='irfanuddin',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author_email='irfanudeen08@gmail.com',
    description='A middleware to log all requests in FastAPI'
)



import json
import pprint
import asyncio
from . import QueueBackend


class PrintBackend(QueueBackend):

    async def send_message(self, message: str):
        await asyncio.sleep(4)
        pprint.pprint(json.loads(message))
